# Copyright (c) 2017, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.
import base64
import bz2
import zlib

from mixbox import entities
from mixbox import fields
from mixbox.vendor import six
from mixbox.compat import xor

import cybox.bindings.artifact_object as artifact_binding
from cybox.common import ObjectProperties, String, HashList


def validate_artifact_type(instance, value):
    if value is None:
        return
    elif value in Artifact.TYPES:
        return
    else:
        err = "Type must be one of %s. Received '%s'." % (Artifact.TYPES, value)
        raise ValueError(err)


def validate_byte_order_endianness(instance, value):
    if value is None:
        return
    elif value in RawArtifact.ENDIANNESS:
        return
    else:
        err = "Type must be one of %s. Received '%s'." % (RawArtifact.ENDIANNESS, value)
        raise ValueError(err)


class RawArtifact(String):
    _binding = artifact_binding
    _binding_class = _binding.RawArtifactType
    _namespace = 'http://cybox.mitre.org/objects#ArtifactObject-2'

    BIG_ENDIAN = "Big-endian"
    LITTLE_ENDIAN = "Little-endian"
    MIDDLE_ENDIAN = "Middle-endian"
    ENDIANNESS = (BIG_ENDIAN, LITTLE_ENDIAN, MIDDLE_ENDIAN)

    byte_order = fields.TypedField("byte_order", preset_hook=validate_byte_order_endianness)


class Compression(entities.Entity):
    """A Compression packaging layer

    Currently only zlib and bz2 are supported.
    Also, compression_mechanism_ref is not currently supported.
    """
    _namespace = 'http://cybox.mitre.org/objects#ArtifactObject-2'
    _binding = artifact_binding
    _binding_class = _binding.CompressionType
    _COMPRESSION_TYPE = None  # overridden by subclasses

    compression_mechanism = fields.TypedField("compression_mechanism")
    compression_mechanism_ref = fields.TypedField("compression_mechanism_ref")

    def __init__(self, compression_mechanism=None, compression_mechanism_ref=None):
        super(Compression, self).__init__()
        self.compression_mechanism = compression_mechanism
        self.compression_mechanism_ref = compression_mechanism_ref

    def pack(self, data):
        """This should accept byte data and return byte data"""
        raise NotImplementedError()

    def unpack(self, packed_data):
        """This should accept byte data and return byte data"""
        raise NotImplementedError()


class Encryption(entities.Entity):
    """
    An encryption packaging layer.
    """
    _namespace = 'http://cybox.mitre.org/objects#ArtifactObject-2'
    _binding = artifact_binding
    _binding_class = _binding.EncryptionType
    _ENCRYPTION_TYPE = None  # overridden by subclasses

    encryption_mechanism = fields.TypedField("encryption_mechanism")
    encryption_mechanism_ref = fields.TypedField("encryption_mechanism_ref")
    encryption_key = fields.TypedField("encryption_key")
    encryption_key_ref = fields.TypedField("encryption_key_ref")

    def __init__(self, encryption_mechanism=None, encryption_key=None,
                 encryption_mechanism_ref=None, encryption_key_ref=None):
        super(Encryption, self).__init__()
        self.encryption_mechanism = encryption_mechanism
        self.encryption_key = encryption_key
        self.encryption_mechanism_ref = encryption_mechanism_ref
        self.encryption_key_ref = encryption_key_ref

    def pack(self, data):
        """This should accept byte data and return byte data"""
        raise NotImplementedError()

    def unpack(self, packed_data):
        """This should accept byte data and return byte data"""
        raise NotImplementedError()


class Encoding(entities.Entity):
    """
    An encoding packaging layer.

    Currently only base64 with a standard alphabet is supported.
    """
    _binding = artifact_binding
    _binding_class = _binding.EncodingType
    _ENCODING_TYPE = None  # overridden by subclasses

    algorithm = fields.TypedField("algorithm")
    character_set = fields.TypedField("character_set")
    custom_character_set_ref = fields.TypedField("custom_character_set_ref")

    def __init__(self, algorithm=None, character_set=None, custom_character_set_ref=None):
        super(Encoding, self).__init__()
        self.algorithm = algorithm
        self.character_set = character_set
        self.custom_character_set_ref = custom_character_set_ref

    def pack(self, data):
        """This should accept byte data and return byte data"""
        raise NotImplementedError()

    def unpack(self, packed_data):
        """This should accept byte data and return byte data"""
        raise NotImplementedError()


class EncryptionFactory(entities.EntityFactory):
    _ENCRYPTION_EXT_MAP = {}

    @classmethod
    def entity_class(cls, key):
        return cls._ENCRYPTION_EXT_MAP.get(key, Encryption)

    @classmethod
    def dictkey(cls, mapping):
        return mapping.get("encryption_mechanism")

    @classmethod
    def objkey(cls, obj):
        return obj.encryption_mechanism

    @classmethod
    def register_extension(cls, new_cls):
        cls._ENCRYPTION_EXT_MAP[new_cls._ENCRYPTION_TYPE] = new_cls
        return new_cls


class CompressionFactory(entities.EntityFactory):
    _COMPRESSION_EXT_MAP = {}

    @classmethod
    def entity_class(cls, key):
        return cls._COMPRESSION_EXT_MAP.get(key, Compression)

    @classmethod
    def dictkey(cls, mapping):
        return mapping.get("compression_mechanism")

    @classmethod
    def objkey(cls, obj):
        return obj.compression_mechanism

    @classmethod
    def register_extension(cls, new_cls):
        cls._COMPRESSION_EXT_MAP[new_cls._COMPRESSION_TYPE] = new_cls
        return new_cls


class EncodingFactory(entities.EntityFactory):
    _ENCODING_EXT_MAP = {}

    @classmethod
    def entity_class(cls, key):
        return cls._ENCODING_EXT_MAP.get(key, Encoding)

    @classmethod
    def dictkey(cls, mapping):
        return mapping.get("algorithm", "Base64")  # default is Base64

    @classmethod
    def objkey(cls, obj):
        return getattr(obj, "algorithm", "Base64")  # default is Base64

    @classmethod
    def register_extension(cls, new_cls):
        cls._ENCODING_EXT_MAP[new_cls._ENCODING_TYPE] = new_cls
        return new_cls


@CompressionFactory.register_extension
class ZlibCompression(Compression):
    _COMPRESSION_TYPE = "zlib"

    def __init__(self):
        super(ZlibCompression, self).__init__(compression_mechanism="zlib")

    def pack(self, data):
        return zlib.compress(data)

    def unpack(self, packed_data):
        return zlib.decompress(packed_data)


@CompressionFactory.register_extension
class Bz2Compression(Compression):
    _COMPRESSION_TYPE = "bz2"

    def __init__(self):
        super(Bz2Compression, self).__init__(compression_mechanism="bz2")

    def pack(self, data):
        return bz2.compress(data)

    def unpack(self, packed_data):
        return bz2.decompress(packed_data)


@EncryptionFactory.register_extension
class XOREncryption(Encryption):
    _ENCRYPTION_TYPE = "xor"

    def __init__(self, key=None):
        super(XOREncryption, self).__init__(
            encryption_mechanism="xor",
            encryption_key=key
        )

    def pack(self, data):
        return xor(data, self.encryption_key)

    def unpack(self, packed_data):
        return xor(packed_data, self.encryption_key)


@EncryptionFactory.register_extension
class PasswordProtectedZipEncryption(Encryption):
    _ENCRYPTION_TYPE = "PasswordProtected"

    def __init__(self, key=None):
        super(PasswordProtectedZipEncryption, self).__init__(
            encryption_mechanism="PasswordProtected",
            encryption_key=key
        )

    # `pack` is not implemented

    def unpack(self, packed_data):
        from zipfile import ZipFile

        buf = six.StringIO(packed_data)
        with ZipFile(buf, 'r') as myzip:
            # Assume there is only one member in the archive, and that it
            # contains the artifact data. Ignore the name.
            filename = myzip.namelist()[0]
            data = myzip.read(filename, self.encryption_key)

        return data


@EncodingFactory.register_extension
class Base64Encoding(Encoding):
    _ENCODING_TYPE = "Base64"

    def __init__(self):
        super(Base64Encoding, self).__init__(algorithm="Base64")

    def pack(self, data):
        return base64.b64encode(data)

    def unpack(self, packed_data):
        return base64.b64decode(packed_data)


class Packaging(entities.Entity):
    """An individual packaging layer."""
    _namespace = 'http://cybox.mitre.org/objects#ArtifactObject-2'
    _binding = artifact_binding
    _binding_class = _binding.PackagingType

    is_encrypted = fields.BooleanField("is_encrypted")
    is_compressed = fields.BooleanField("is_compressed")
    compression = fields.TypedField("Compression", Compression, factory=CompressionFactory, multiple=True)
    encryption = fields.TypedField("Encryption", Encryption, factory=EncryptionFactory, multiple=True)
    encoding = fields.TypedField("Encoding", Encoding, factory=EncodingFactory, multiple=True)

    def __init__(self, is_encrypted=None, is_compressed=None, compression=None, encryption=None, encoding=None):
        super(Packaging, self).__init__()
        self.is_encrypted = is_encrypted
        self.is_compressed = is_compressed
        self.compression = compression
        self.encryption = encryption
        self.encoding = encoding


class Artifact(ObjectProperties):
    # Warning: Do not attempt to get or set Raw_Artifact directly. Use `data`
    # or `packed_data` respectively. The Raw_Artifact value will be set on
    # export. You can set BaseObjectProperties or PatternFieldGroup attributes.
    _binding = artifact_binding
    _binding_class = _binding.ArtifactObjectType
    _namespace = 'http://cybox.mitre.org/objects#ArtifactObject-2'
    _XSI_NS = "ArtifactObj"
    _XSI_TYPE = "ArtifactObjectType"

    TYPE_FILE = "File"
    TYPE_MEMORY = "Memory Region"
    TYPE_FILE_SYSTEM = "File System Fragment"
    TYPE_NETWORK = "Network Traffic"
    TYPE_GENERIC = "Generic Data Region"
    TYPES = (TYPE_FILE, TYPE_FILE_SYSTEM, TYPE_GENERIC, TYPE_MEMORY, TYPE_NETWORK)

    hashes = fields.TypedField("Hashes", HashList)
    packaging = fields.TypedField("Packaging", Packaging)
    type_ = fields.TypedField("type_", key_name="type", preset_hook=validate_artifact_type)
    content_type = fields.TypedField("content_type")
    content_type_version = fields.TypedField("content_type_version")
    suspected_malicious = fields.TypedField("suspected_malicious")
    # TODO: xs:choice
    raw_artifact = fields.TypedField("Raw_Artifact", RawArtifact)
    raw_artifact_reference = fields.TypedField("Raw_Artifact_Reference")

    def __init__(self, data=None, type_=None):
        super(Artifact, self).__init__()
        self.type_ = type_

        # `data` is the actual binary data that is being encoded in this
        # Artifact. It should use the `str` type on Python 2 or the `bytes`
        # type on Python 3.

        # `packed_data` is the literal character data that comes from (or
        # becomes) the contents of the Raw_Artifact element. It should be a
        # Unicode string (`unicode` on Python 2, `str` on Python 3), and should
        # in general be ASCII-encoded, since any other data should be
        # Base64-encoded.

        # Only one of these two attributes can be set directly. The other can
        # be calculated based on the various `Packaging` types added to this
        # Artifact.

        # We set the private attribute `_packed_data` first, so that the setter
        # for `data` has access to this attribute.
        self._packed_data = None
        self.data = data

    @property
    def data(self):
        """Should return a byte string"""
        if self._data:
            return self._data
        elif self._packed_data:
            tmp_data = self._packed_data.encode('ascii')
            if self.packaging:
                for p in reversed(self.packaging.encoding):
                    tmp_data = p.unpack(tmp_data)
                for p in reversed(self.packaging.encryption):
                    tmp_data = p.unpack(tmp_data)
                for p in reversed(self.packaging.compression):
                    tmp_data = p.unpack(tmp_data)
            return tmp_data
        else:
            return None

    @data.setter
    def data(self, value):
        if self._packed_data:
            raise ValueError("packed_data already set, can't set data")
        if value is not None and not isinstance(value, six.binary_type):
            msg = ("Artifact data must be either None or byte data, not a "
                   "Unicode string.")
            raise ValueError(msg)
        self._data = value

    @property
    def packed_data(self):
        """Should return a Unicode string"""
        if self._packed_data:
            return self._packed_data
        elif self._data:
            tmp_data = self._data
            if self.packaging:
                for p in self.packaging.compression:
                    tmp_data = p.pack(tmp_data)
                for p in self.packaging.encryption:
                    tmp_data = p.pack(tmp_data)
                for p in self.packaging.encoding:
                    tmp_data = p.pack(tmp_data)
            return tmp_data.decode('ascii')
        else:
            return None

    @packed_data.setter
    def packed_data(self, value):
        if self._data:
            raise ValueError("data already set, can't set packed_data")
        if value is not None and not isinstance(value, six.text_type):
            msg = ("Artifact packed_data must be either None or a Unicode "
                   "string, not byte data.")
            raise ValueError(msg)
        self._packed_data = value

    def to_obj(self, ns_info=None):
        artifact_obj = super(Artifact, self).to_obj(ns_info=ns_info)

        if self.packed_data:
            if not self.raw_artifact:
                self.raw_artifact = RawArtifact()
            self.raw_artifact.value = self.packed_data
            artifact_obj.Raw_Artifact = self.raw_artifact.to_obj(ns_info=ns_info)

        return artifact_obj

    def to_dict(self):
        artifact_dict = super(Artifact, self).to_dict()

        if self.packed_data:
            if not self.raw_artifact:
                self.raw_artifact = RawArtifact()
            self.raw_artifact.value = self.packed_data
            artifact_dict['raw_artifact'] = self.raw_artifact.to_dict()

        return artifact_dict

    @classmethod
    def from_obj(cls, cls_obj):
        if not cls_obj:
            return None

        artifact = super(Artifact, cls).from_obj(cls_obj)

        raw_artifact = cls_obj.Raw_Artifact
        if raw_artifact:
            artifact.raw_artifact = RawArtifact.from_obj(raw_artifact)
            artifact.packed_data = six.text_type(artifact.raw_artifact.value)

        return artifact

    @classmethod
    def from_dict(cls, cls_dict):
        if not cls_dict:
            return None

        artifact = super(Artifact, cls).from_dict(cls_dict)

        raw_artifact = cls_dict.get('raw_artifact')
        if raw_artifact:
            artifact.raw_artifact = RawArtifact.from_dict(raw_artifact)
            artifact.packed_data = six.text_type(artifact.raw_artifact.value)

        return artifact



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities, fields

import cybox.bindings.cybox_common as common_binding
from cybox.common.structured_text import StructuredText


class UsageContextAssumptions(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.UsageContextAssumptionsType
    _namespace = 'http://cybox.mitre.org/common-2'

    usage_context_assumption = fields.TypedField("Usage_Context_Assumption", StructuredText, multiple=True)



# Copyright (c) 2017, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.library_object as library_binding
from cybox.common.extracted_features import ExtractedFeatures
from cybox.common import BaseProperty, ObjectProperties, String, UnsignedLong, HexBinary


class LibraryType(BaseProperty):
    _binding = library_binding
    _binding_class = library_binding.LibraryType
    _namespace = "http://cybox.mitre.org/objects#LibraryObject-2"
    default_datatype = "string"

    TYPE_DYNAMIC = "Dynamic"
    TYPE_STATIC = "Static"
    TYPE_REMOTE = "Remote"
    TYPE_SHARED = "Shared"
    TYPE_OTHER = "Other"


class Library(ObjectProperties):
    _binding = library_binding
    _binding_class = library_binding.LibraryObjectType
    _namespace = "http://cybox.mitre.org/objects#LibraryObject-2"
    _XSI_NS = "LibraryObj"
    _XSI_TYPE = "LibraryObjectType"

    name = fields.TypedField("Name", String)
    path = fields.TypedField("Path", String)
    size = fields.TypedField("Size", UnsignedLong)
    type_ = fields.TypedField("Type", LibraryType)
    version = fields.TypedField("Version", String)
    base_address = fields.TypedField("Base_Address", HexBinary)
    extracted_features = fields.TypedField("Extracted_Features", ExtractedFeatures)



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities, fields

import cybox
import cybox.bindings.unix_process_object as unix_process_binding
from cybox.common import BaseProperty, DateTime, NonNegativeInteger, UnsignedInteger
from cybox.objects.process_object import Process, ProcessStatus


class FileDescriptorList(entities.EntityList):
    _binding = unix_process_binding
    _binding_class = unix_process_binding.FileDescriptorListType
    _namespace = "http://cybox.mitre.org/objects#UnixProcessObject-2"

    file_descriptor = fields.TypedField("File_Descriptor", UnsignedInteger, multiple=True)


class UnixProcessState(BaseProperty):
    _binding = unix_process_binding
    _binding_class = unix_process_binding.UnixProcessStateType
    _namespace = "http://cybox.mitre.org/objects#UnixProcessObject-2"

    STATE_RUNNING = "Running"
    STATE_UNINTERRUPTIBLE_SLEEP = "UninterruptibleSleep"
    STATE_INTERRUPTIBLE_SLEEP = "InterruptibleSleep"
    STATE_STOPPED = "Stopped"
    STATE_PAGING = "Paging"
    STATE_DEAD = "Dead"
    STATE_ZOMBIE = "Zombie"


@cybox.register_extension
class UnixProcessStatus(ProcessStatus):
    _binding = unix_process_binding
    _binding_class = unix_process_binding.UnixProcessStatusType
    _namespace = "http://cybox.mitre.org/objects#UnixProcessObject-2"
    _XSI_TYPE = "UnixProcessObj:UnixProcessStatusType"

    current_status = fields.TypedField("Current_Status", UnixProcessState)
    timestamp = fields.TypedField("Timestamp", DateTime)


class UnixProcess(Process):
    _binding = unix_process_binding
    _binding_class = unix_process_binding.UnixProcessObjectType
    _namespace = "http://cybox.mitre.org/objects#UnixProcessObject-2"
    _XSI_NS = "UnixProcessObj"
    _XSI_TYPE = "UnixProcessObjectType"

    open_file_descriptor_list = fields.TypedField("Open_File_Descriptor_List", FileDescriptorList)
    priority = fields.TypedField("Priority", NonNegativeInteger)
    ruid = fields.TypedField("RUID", NonNegativeInteger)
    session_id = fields.TypedField("Session_ID", NonNegativeInteger)



# Copyright (c) 2017, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.core import Event, Observable
from cybox.test import EntityTestCase


class TestEvent(EntityTestCase, unittest.TestCase):
    klass = Event

    _full_dict = {
        'id': "example:Event-1",
        'idref': "example:Event-2",
        'type': u("Port Scan"),
        'description': u("This is a test event"),
        'observation_method': {'class': "Network"},
        'frequency': {'rate': 1.0},
        'actions': [
            {'idref': "example:Action-5"},
            {'idref': "example:Action-6"},
        ],
        'location': {
            'name': "Some location",
            'id': "example:Location-A"
        },
        # Once the choice is implemented, this won't work
        'event': [
            {'idref': "example:Event-A"},
            {'idref': "example:Event-B"},
        ]
    }

    def test_empty_recursive_event(self):
        e = Event()
        e.description = "Foo"
        e.event = None
        self.assertTrue(b"Foo" in e.to_xml())

    def test_observable_from_dict_with_event(self):
        data = {
            'event': {
                'type': {
                    'xsi:type': 'cyboxVocabs:EventTypeVocab-1.0.1',
                    'value': 'DHCP',
                }
            }
        }
        obs = Observable.from_dict(data)
        assert obs.event.type_ == 'DHCP'



if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2017, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.archive_file_object as archive_binding
from cybox.common import BaseProperty, Integer, String
from cybox.common.cipher import Cipher
from cybox.objects.file_object import File


class ArchiveFileFormat(BaseProperty):
    _binding = archive_binding
    _binding_class = archive_binding.ArchiveFileFormatType
    _namespace = 'http://cybox.mitre.org/objects#ArchiveFileObject-1'

    TERM_7ZIP = "7-ZIP"
    TERM_APK = "APK"
    TERM_CAB = "CAB"
    TERM_DMG = "DMG"
    TERM_JAR = "JAR"
    TERM_RAR = "RAR"
    TERM_SIT = "SIT"
    TERM_TGZ = "TGZ"
    TERM_ZIP = "ZIP"


class ArchiveFile(File):
    _binding = archive_binding
    _binding_class = archive_binding.ArchiveFileObjectType
    _namespace = 'http://cybox.mitre.org/objects#ArchiveFileObject-1'
    _XSI_NS = "ArchiveFileObj"
    _XSI_TYPE = "ArchiveFileObjectType"

    archive_format = fields.TypedField("Archive_Format", ArchiveFileFormat)
    version = fields.TypedField("Version", String)
    file_count = fields.TypedField("File_Count", Integer)
    encryption_algorithm = fields.TypedField("Encryption_Algorithm", Cipher)
    decryption_key = fields.TypedField("Decryption_Key", String)
    comment = fields.TypedField("Comment", String)
    archived_file = fields.TypedField("Archived_File", File, multiple=True)



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities, fields

import cybox.bindings.url_history_object as url_binding
from cybox.common import (
    DateTime, NonNegativeInteger, ObjectProperties, String, ToolInformation
)
from cybox.objects.hostname_object import Hostname
from cybox.objects.uri_object import URI


class URLHistoryEntry(entities.Entity):
    _binding = url_binding
    _binding_class = url_binding.URLHistoryEntryType
    _namespace = "http://cybox.mitre.org/objects#URLHistoryObject-1"

    url = fields.TypedField("URL", URI)
    hostname = fields.TypedField("Hostname", Hostname)
    referrer_url = fields.TypedField("Referrer_URL", URI)
    page_title = fields.TypedField("Page_Title", String)
    user_profile_name = fields.TypedField("User_Profile_Name", String)
    visit_count = fields.TypedField("Visit_Count", NonNegativeInteger)
    manually_entered_count = fields.TypedField("Manually_Entered_Count", NonNegativeInteger)
    modification_datetime = fields.TypedField("Modification_DateTime", DateTime)
    expiration_datetime = fields.TypedField("Expiration_DateTime", DateTime)
    first_visit_datetime = fields.TypedField("First_Visit_DateTime", DateTime)
    last_visit_datetime = fields.TypedField("Last_Visit_DateTime", DateTime)


class URLHistory(ObjectProperties):
    _binding = url_binding
    _binding_class = url_binding.URLHistoryObjectType
    _namespace = "http://cybox.mitre.org/objects#URLHistoryObject-1"
    _XSI_NS = "URLHistoryObj"
    _XSI_TYPE = "URLHistoryObjectType"

    browser_information = fields.TypedField("Browser_Information", ToolInformation)
    url_history_entry = fields.TypedField("URL_History_Entry", URLHistoryEntry, multiple=True)



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities, fields

import cybox.bindings.cybox_common as common_binding
from cybox.common.compilers import Compilers
from cybox.common.configuration_settings import ConfigurationSettings
from cybox.common.libraries import Libraries
from cybox.common.platform_specification import PlatformSpecification
from cybox.common.time import DateTimeWithPrecision


class BuildUtility(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.BuildUtilityType
    _namespace = 'http://cybox.mitre.org/common-2'

    build_utility_name = fields.TypedField("Build_Utility_Name")
    build_utility_platform_specification = fields.TypedField("Build_Utility_Platform_Specification", PlatformSpecification)


class BuildConfiguration(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.BuildConfigurationType
    _namespace = 'http://cybox.mitre.org/common-2'

    configuration_setting_description = fields.TypedField("Configuration_Setting_Description", multiple=True)
    configuration_settings = fields.TypedField("Configuration_Settings", ConfigurationSettings)


class BuildInformation(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.BuildInformationType
    _namespace = 'http://cybox.mitre.org/common-2'

    build_id = fields.TypedField("Build_ID")
    build_project = fields.TypedField("Build_Project")
    build_utility = fields.TypedField("Build_Utility", BuildUtility)
    build_version = fields.TypedField("Build_Version")
    build_label = fields.TypedField("Build_Label")
    compilers = fields.TypedField("Compilers", Compilers)
    compilation_date = fields.TypedField("Compilation_Date", DateTimeWithPrecision)
    build_configuration = fields.TypedField("Build_Configuration", BuildConfiguration)
    build_script = fields.TypedField("Build_Script")
    libraries = fields.TypedField("Libraries", Libraries)
    build_output_log = fields.TypedField("Build_Output_Log")



# Copyright (c) 2017, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
from . import network_route_entry_object


class NetworkRouteEntriesType(GeneratedsSuper):
    """The NetworkRouteEntriesType type is intended to characterize the set
    of network route segments for this route."""

    subclass = None
    superclass = None
    def __init__(self, Network_Route_Entry=None):
        if Network_Route_Entry is None:
            self.Network_Route_Entry = []
        else:
            self.Network_Route_Entry = Network_Route_Entry
    def factory(*args_, **kwargs_):
        if NetworkRouteEntriesType.subclass:
            return NetworkRouteEntriesType.subclass(*args_, **kwargs_)
        else:
            return NetworkRouteEntriesType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Network_Route_Entry(self): return self.Network_Route_Entry
    def set_Network_Route_Entry(self, Network_Route_Entry): self.Network_Route_Entry = Network_Route_Entry
    def add_Network_Route_Entry(self, value): self.Network_Route_Entry.append(value)
    def insert_Network_Route_Entry(self, index, value): self.Network_Route_Entry[index] = value
    def hasContent_(self):
        if (
            self.Network_Route_Entry
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='NetworkRouteObj:', name_='NetworkRouteEntriesType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='NetworkRouteEntriesType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='NetworkRouteObj:', name_='NetworkRouteEntriesType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='NetworkRouteObj:', name_='NetworkRouteEntriesType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for Network_Route_Entry_ in self.Network_Route_Entry:
            Network_Route_Entry_.export(lwrite, level, 'NetworkRouteObj:', name_='Network_Route_Entry', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Network_Route_Entry':
            obj_ = network_route_entry_object.NetworkRouteEntryObjectType.factory()
            obj_.build(child_)
            self.Network_Route_Entry.append(obj_)
# end class NetworkRouteEntriesType

class NetRouteObjectType(cybox_common.ObjectPropertiesType):
    """The NetRouteObjectType type is intended to characterize a specific
    network route.The is_ipv6 field specifies whether or not the
    route uses IPv6 addresses.The is_autoconfigure_address field
    specifies if the IP address is autoconfigured.The is_immortal
    field specifies if the route is immortal.The is_loopback field
    specifies if the route is a loopback route (the gateway is on
    the local host).The is_publish field specifies if the route is
    published."""

    subclass = None
    superclass = cybox_common.ObjectPropertiesType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, is_publish=None, is_autoconfigure_address=None, is_loopback=None, is_immortal=None, is_ipv6=None, Description=None, Network_Route_Entries=None, Preferred_Lifetime=None, Valid_Lifetime=None, Route_Age=None):
        super(NetRouteObjectType, self).__init__(object_reference, Custom_Properties, xsi_type )
        self.is_publish = _cast(bool, is_publish)
        self.is_autoconfigure_address = _cast(bool, is_autoconfigure_address)
        self.is_loopback = _cast(bool, is_loopback)
        self.is_immortal = _cast(bool, is_immortal)
        self.is_ipv6 = _cast(bool, is_ipv6)
        self.Description = Description
        self.Network_Route_Entries = Network_Route_Entries
        self.Preferred_Lifetime = Preferred_Lifetime
        self.Valid_Lifetime = Valid_Lifetime
        self.Route_Age = Route_Age
    def factory(*args_, **kwargs_):
        if NetRouteObjectType.subclass:
            return NetRouteObjectType.subclass(*args_, **kwargs_)
        else:
            return NetRouteObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Description(self): return self.Description
    def set_Description(self, Description): self.Description = Description
    def get_Network_Route_Entries(self): return self.Network_Route_Entries
    def set_Network_Route_Entries(self, Network_Route_Entries): self.Network_Route_Entries = Network_Route_Entries
    def get_Preferred_Lifetime(self): return self.Preferred_Lifetime
    def set_Preferred_Lifetime(self, Preferred_Lifetime): self.Preferred_Lifetime = Preferred_Lifetime
    def validate_DurationObjectPropertyType(self, value):
        # Validate type cybox_common.DurationObjectPropertyType, a restriction on None.
        pass
    def get_Valid_Lifetime(self): return self.Valid_Lifetime
    def set_Valid_Lifetime(self, Valid_Lifetime): self.Valid_Lifetime = Valid_Lifetime
    def get_Route_Age(self): return self.Route_Age
    def set_Route_Age(self, Route_Age): self.Route_Age = Route_Age
    def get_is_publish(self): return self.is_publish
    def set_is_publish(self, is_publish): self.is_publish = is_publish
    def get_is_autoconfigure_address(self): return self.is_autoconfigure_address
    def set_is_autoconfigure_address(self, is_autoconfigure_address): self.is_autoconfigure_address = is_autoconfigure_address
    def get_is_loopback(self): return self.is_loopback
    def set_is_loopback(self, is_loopback): self.is_loopback = is_loopback
    def get_is_immortal(self): return self.is_immortal
    def set_is_immortal(self, is_immortal): self.is_immortal = is_immortal
    def get_is_ipv6(self): return self.is_ipv6
    def set_is_ipv6(self, is_ipv6): self.is_ipv6 = is_ipv6
    def hasContent_(self):
        if (
            self.Description is not None or
            self.Network_Route_Entries is not None or
            self.Preferred_Lifetime is not None or
            self.Valid_Lifetime is not None or
            self.Route_Age is not None or
            super(NetRouteObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='NetworkRouteObj:', name_='NetRouteObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='NetRouteObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='NetworkRouteObj:', name_='NetRouteObjectType'):
        super(NetRouteObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='NetRouteObjectType')
        if self.is_publish is not None:

            lwrite(' is_publish="%s"' % self.gds_format_boolean(self.is_publish, input_name='is_publish'))
        if self.is_autoconfigure_address is not None:

            lwrite(' is_autoconfigure_address="%s"' % self.gds_format_boolean(self.is_autoconfigure_address, input_name='is_autoconfigure_address'))
        if self.is_loopback is not None:

            lwrite(' is_loopback="%s"' % self.gds_format_boolean(self.is_loopback, input_name='is_loopback'))
        if self.is_immortal is not None:

            lwrite(' is_immortal="%s"' % self.gds_format_boolean(self.is_immortal, input_name='is_immortal'))
        if self.is_ipv6 is not None:

            lwrite(' is_ipv6="%s"' % self.gds_format_boolean(self.is_ipv6, input_name='is_ipv6'))
    def exportChildren(self, lwrite, level, namespace_='NetworkRouteObj:', name_='NetRouteObjectType', fromsubclass_=False, pretty_print=True):
        super(NetRouteObjectType, self).exportChildren(lwrite, level, 'NetworkRouteObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Description is not None:
            self.Description.export(lwrite, level, 'NetworkRouteObj:', name_='Description', pretty_print=pretty_print)
        if self.Network_Route_Entries is not None:
            self.Network_Route_Entries.export(lwrite, level, 'NetworkRouteObj:', name_='Network_Route_Entries', pretty_print=pretty_print)
        if self.Preferred_Lifetime is not None:
            self.Preferred_Lifetime.export(lwrite, level, 'NetworkRouteObj:', name_='Preferred_Lifetime', pretty_print=pretty_print)
        if self.Valid_Lifetime is not None:
            self.Valid_Lifetime.export(lwrite, level, 'NetworkRouteObj:', name_='Valid_Lifetime', pretty_print=pretty_print)
        if self.Route_Age is not None:
            self.Route_Age.export(lwrite, level, 'NetworkRouteObj:', name_='Route_Age', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('is_publish', node)
        if value is not None:

            if value in ('true', '1'):
                self.is_publish = True
            elif value in ('false', '0'):
                self.is_publish = False
            else:
                raise_parse_error(node, 'Bad boolean attribute')
        value = find_attr_value_('is_autoconfigure_address', node)
        if value is not None:

            if value in ('true', '1'):
                self.is_autoconfigure_address = True
            elif value in ('false', '0'):
                self.is_autoconfigure_address = False
            else:
                raise_parse_error(node, 'Bad boolean attribute')
        value = find_attr_value_('is_loopback', node)
        if value is not None:

            if value in ('true', '1'):
                self.is_loopback = True
            elif value in ('false', '0'):
                self.is_loopback = False
            else:
                raise_parse_error(node, 'Bad boolean attribute')
        value = find_attr_value_('is_immortal', node)
        if value is not None:

            if value in ('true', '1'):
                self.is_immortal = True
            elif value in ('false', '0'):
                self.is_immortal = False
            else:
                raise_parse_error(node, 'Bad boolean attribute')
        value = find_attr_value_('is_ipv6', node)
        if value is not None:

            if value in ('true', '1'):
                self.is_ipv6 = True
            elif value in ('false', '0'):
                self.is_ipv6 = False
            else:
                raise_parse_error(node, 'Bad boolean attribute')
        super(NetRouteObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Description':
            obj_ = cybox_common.StructuredTextType.factory()
            obj_.build(child_)
            self.set_Description(obj_)
        elif nodeName_ == 'Network_Route_Entries':
            obj_ = NetworkRouteEntriesType.factory()
            obj_.build(child_)
            self.set_Network_Route_Entries(obj_)
        elif nodeName_ == 'Preferred_Lifetime':
            obj_ = cybox_common.DurationObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Preferred_Lifetime(obj_)
        elif nodeName_ == 'Valid_Lifetime':
            obj_ = cybox_common.DurationObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Valid_Lifetime(obj_)
        elif nodeName_ == 'Route_Age':
            obj_ = cybox_common.DurationObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Route_Age(obj_)
        super(NetRouteObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class NetRouteObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Errors': cybox_common.ErrorsType,
    'Time': cybox_common.TimeType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Route_Age': cybox_common.DurationObjectPropertyType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Protocol': cybox_common.StringObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Type': network_route_entry_object.RouteType,
    'Compilers': cybox_common.CompilersType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'Tool': cybox_common.ToolInformationType,
    'Preferred_Lifetime': cybox_common.DurationObjectPropertyType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'Interface': cybox_common.StringObjectPropertyType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Language': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Metric': cybox_common.UnsignedLongObjectPropertyType,
    'Reference_Description': cybox_common.StructuredTextType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Network_Route_Entry': network_route_entry_object.NetworkRouteEntryObjectType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Import': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'Contributors': cybox_common.PersonnelType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Libraries': cybox_common.LibrariesType,
    'Function': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'VLAN_Name': cybox_common.StringObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'Compiler': cybox_common.CompilerType,
    'Name': cybox_common.StringObjectPropertyType,
    'Valid_Lifetime': cybox_common.DurationObjectPropertyType,
    'Address_Value': cybox_common.StringObjectPropertyType,
    'VLAN_Num': cybox_common.IntegerObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'Contributor': cybox_common.ContributorType,
    'Tools': cybox_common.ToolsInformationType,
    'Data_Size': cybox_common.DataSizeType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Network_Route_Object'
        rootClass = NetRouteObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Network_Route_Object'
        rootClass = NetRouteObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Network_Route_Object'
        rootClass = NetRouteObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="Network_Route_Object",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "NetRouteObjectType",
    "NetworkRouteEntriesType"
    ]



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.library_object import Library
from cybox.test.objects import ObjectTestCase


class TestLibrary(ObjectTestCase, unittest.TestCase):
    object_type = "LibraryObjectType"
    klass = Library

    _full_dict = {
        'name': u('libc'),
        'path': u('/lib/x86_64-linux-gnu/'),
        'size': 28000,
        'type': u('Dynamic'),
        'version': u('2.19'),
        'base_address': u('0x7ffff7bcf000'),
        # 'extracted_features': '',
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2017, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities, fields

import cybox.bindings.cybox_core as core_binding
from cybox.common import Property
from cybox.core import Action, Event, Object


class EventPool(entities.Entity):
    _binding = core_binding
    _binding_class = _binding.EventPoolType
    _namespace = 'http://cybox.mitre.org/cybox-2'

    events = fields.TypedField("Event", Event, multiple=True, key_name="events")


class ActionPool(entities.Entity):
    _binding = core_binding
    _binding_class = _binding.ActionPoolType
    _namespace = 'http://cybox.mitre.org/cybox-2'

    actions = fields.TypedField("Action", Action, multiple=True, key_name="actions")


class ObjectPool(entities.Entity):
    _binding = core_binding
    _binding_class = _binding.ObjectPoolType
    _namespace = 'http://cybox.mitre.org/cybox-2'

    objects = fields.TypedField("Object", Object, multiple=True, key_name="objects")


class PropertyPool(entities.Entity):
    _binding = core_binding
    _binding_class = _binding.PropertyPoolType
    _namespace = 'http://cybox.mitre.org/cybox-2'

    properties = fields.TypedField("Property", Property, multiple=True, key_name="properties")


class Pools(entities.Entity):
    _binding = core_binding
    _binding_class = _binding.PoolsType
    _namespace = 'http://cybox.mitre.org/cybox-2'

    event_pool = fields.TypedField("Event_Pool", EventPool)
    action_pool = fields.TypedField("Action_Pool", ActionPool)
    object_pool = fields.TypedField("Object_Pool", ObjectPool)
    property_pool = fields.TypedField("Property_Pool", PropertyPool)



# Copyright (c) 2017, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
from . import volume_object


class WindowsVolumeAttributesListType(GeneratedsSuper):
    """A list of attributes describing this windows volume."""

    subclass = None
    superclass = None
    def __init__(self, Attribute=None):
        if Attribute is None:
            self.Attribute = []
        else:
            self.Attribute = Attribute
    def factory(*args_, **kwargs_):
        if WindowsVolumeAttributesListType.subclass:
            return WindowsVolumeAttributesListType.subclass(*args_, **kwargs_)
        else:
            return WindowsVolumeAttributesListType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Attribute(self): return self.Attribute
    def set_Attribute(self, Attribute): self.Attribute = Attribute
    def add_Attribute(self, value): self.Attribute.append(value)
    def insert_Attribute(self, index, value): self.Attribute[index] = value
    def validate_WindowsVolumeAttributeType(self, value):
        # Validate type WindowsVolumeAttributeType, a restriction on None.
        pass
    def hasContent_(self):
        if (
            self.Attribute
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinVolumeObj:', name_='WindowsVolumeAttributesListType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsVolumeAttributesListType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinVolumeObj:', name_='WindowsVolumeAttributesListType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='WinVolumeObj:', name_='WindowsVolumeAttributesListType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for Attribute_ in self.Attribute:
            Attribute_.export(lwrite, level, 'WinVolumeObj:', name_='Attribute', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Attribute':
            obj_ = WindowsVolumeAttributeType.factory()
            obj_.build(child_)
            self.Attribute.append(obj_)
# end class WindowsVolumeAttributesListType

class WindowsVolumeAttributeType(cybox_common.BaseObjectPropertyType):
    """WindowsVolumeAttributeType specifies Windows volume attributes via a
    union of the WindowsVolumeAttributeEnum type and the atomic
    xs:string type. Its base type is the CybOX Core
    cybox_common.BaseObjectPropertyType, for permitting complex (i.e. regular-
    expression based) specifications.This attribute is optional and
    specifies the expected type for the value of the specified
    property."""

    subclass = None
    superclass = cybox_common.BaseObjectPropertyType
    def __init__(self, obfuscation_algorithm_ref=None, refanging_transform_type=None, has_changed=None, delimiter='##comma##', pattern_type=None, datatype='string', refanging_transform=None, is_case_sensitive=True, bit_mask=None, appears_random=None, observed_encoding=None, defanging_algorithm_ref=None, is_obfuscated=None, regex_syntax=None, apply_condition='ANY', trend=None, idref=None, is_defanged=None, id=None, condition=None, valueOf_=None):
        super(WindowsVolumeAttributeType, self).__init__(obfuscation_algorithm_ref, refanging_transform_type, has_changed, delimiter, pattern_type, datatype, refanging_transform, is_case_sensitive, bit_mask, appears_random, observed_encoding, defanging_algorithm_ref, is_obfuscated, regex_syntax, apply_condition, trend, idref, is_defanged, id, condition, valueOf_)
        self.datatype = _cast(None, datatype)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if WindowsVolumeAttributeType.subclass:
            return WindowsVolumeAttributeType.subclass(*args_, **kwargs_)
        else:
            return WindowsVolumeAttributeType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_datatype(self): return self.datatype
    def set_datatype(self, datatype): self.datatype = datatype
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_ or
            super(WindowsVolumeAttributeType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinVolumeObj:', name_='WindowsVolumeAttributeType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsVolumeAttributeType')
        if self.hasContent_():
            lwrite('>')
            lwrite(quote_xml(self.valueOf_))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinVolumeObj:', name_='WindowsVolumeAttributeType'):
        super(WindowsVolumeAttributeType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsVolumeAttributeType')
        if self.datatype is not None:

            lwrite(' datatype=%s' % (quote_attrib(self.datatype), ))
    def exportChildren(self, lwrite, level, namespace_='WinVolumeObj:', name_='WindowsVolumeAttributeType', fromsubclass_=False, pretty_print=True):
        super(WindowsVolumeAttributeType, self).exportChildren(lwrite, level, 'WinVolumeObj:', name_, True, pretty_print=pretty_print)
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('datatype', node)
        if value is not None:

            self.datatype = value
        super(WindowsVolumeAttributeType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class WindowsVolumeAttributeType

class WindowsDriveType(cybox_common.BaseObjectPropertyType):
    """WindowsDriveType specifies Windows drive types via a union of the
    WindowsDriveTypeEnum type and the atomic xs:string type. Its
    base type is the CybOX Core cybox_common.BaseObjectPropertyType, for
    permitting complex (i.e. regular-expression based)
    specifications.This attribute is optional and specifies the
    expected type for the value of the specified property."""

    subclass = None
    superclass = cybox_common.BaseObjectPropertyType
    def __init__(self, obfuscation_algorithm_ref=None, refanging_transform_type=None, has_changed=None, delimiter='##comma##', pattern_type=None, datatype='string', refanging_transform=None, is_case_sensitive=True, bit_mask=None, appears_random=None, trend=None, defanging_algorithm_ref=None, is_obfuscated=None, regex_syntax=None, apply_condition='ANY', idref=None, is_defanged=None, id=None, condition=None, valueOf_=None):
        super(WindowsDriveType, self).__init__(obfuscation_algorithm_ref, refanging_transform_type, has_changed, delimiter, pattern_type, datatype, refanging_transform, is_case_sensitive, bit_mask, appears_random, trend, defanging_algorithm_ref, is_obfuscated, regex_syntax, apply_condition, idref, is_defanged, id, condition, valueOf_, )
        self.datatype = _cast(None, datatype)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if WindowsDriveType.subclass:
            return WindowsDriveType.subclass(*args_, **kwargs_)
        else:
            return WindowsDriveType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_datatype(self): return self.datatype
    def set_datatype(self, datatype): self.datatype = datatype
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_ or
            super(WindowsDriveType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinVolumeObj:', name_='WindowsDriveType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsDriveType')
        if self.hasContent_():
            lwrite('>')
            lwrite(quote_xml(self.valueOf_))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinVolumeObj:', name_='WindowsDriveType'):
        super(WindowsDriveType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsDriveType')
        if self.datatype is not None:

            lwrite(' datatype=%s' % (quote_attrib(self.datatype), ))
    def exportChildren(self, lwrite, level, namespace_='WinVolumeObj:', name_='WindowsDriveType', fromsubclass_=False, pretty_print=True):
        super(WindowsDriveType, self).exportChildren(lwrite, level, 'WinVolumeObj:', name_, True, pretty_print=pretty_print)
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('datatype', node)
        if value is not None:

            self.datatype = value
        super(WindowsDriveType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class WindowsDriveType

class WindowsVolumeObjectType(volume_object.VolumeObjectType):
    """The WindowsVolumeObjectType type is intended to characterize Windows
    disk volumes."""

    subclass = None
    superclass = volume_object.VolumeObjectType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, is_mounted=None, Name=None, Device_Path=None, File_System_Type=None, Total_Allocation_Units=None, Sectors_Per_Allocation_Unit=None, Bytes_Per_Sector=None, Actual_Available_Allocation_Units=None, Creation_Time=None, File_System_Flag_List=None, Serial_Number=None, Attributes_List=None, Drive_Letter=None, Drive_Type=None):
        super(WindowsVolumeObjectType, self).__init__(object_reference, Custom_Properties, is_mounted, Name, Device_Path, File_System_Type, Total_Allocation_Units, Sectors_Per_Allocation_Unit, Bytes_Per_Sector, Actual_Available_Allocation_Units, Creation_Time, File_System_Flag_List, Serial_Number, )
        self.Attributes_List = Attributes_List
        self.Drive_Letter = Drive_Letter
        self.Drive_Type = Drive_Type
    def factory(*args_, **kwargs_):
        if WindowsVolumeObjectType.subclass:
            return WindowsVolumeObjectType.subclass(*args_, **kwargs_)
        else:
            return WindowsVolumeObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Attributes_List(self): return self.Attributes_List
    def set_Attributes_List(self, Attributes_List): self.Attributes_List = Attributes_List
    def get_Drive_Letter(self): return self.Drive_Letter
    def set_Drive_Letter(self, Drive_Letter): self.Drive_Letter = Drive_Letter
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Drive_Type(self): return self.Drive_Type
    def set_Drive_Type(self, Drive_Type): self.Drive_Type = Drive_Type
    def validate_WindowsDriveType(self, value):
        # Validate type WindowsDriveType, a restriction on None.
        pass
    def hasContent_(self):
        if (
            self.Attributes_List is not None or
            self.Drive_Letter is not None or
            self.Drive_Type is not None or
            super(WindowsVolumeObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinVolumeObj:', name_='WindowsVolumeObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsVolumeObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinVolumeObj:', name_='WindowsVolumeObjectType'):
        super(WindowsVolumeObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsVolumeObjectType')
    def exportChildren(self, lwrite, level, namespace_='WinVolumeObj:', name_='WindowsVolumeObjectType', fromsubclass_=False, pretty_print=True):
        super(WindowsVolumeObjectType, self).exportChildren(lwrite, level, 'WinVolumeObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Attributes_List is not None:
            self.Attributes_List.export(lwrite, level, 'WinVolumeObj:', name_='Attributes_List', pretty_print=pretty_print)
        if self.Drive_Letter is not None:
            self.Drive_Letter.export(lwrite, level, 'WinVolumeObj:', name_='Drive_Letter', pretty_print=pretty_print)
        if self.Drive_Type is not None:
            self.Drive_Type.export(lwrite, level, 'WinVolumeObj:', name_='Drive_Type', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(WindowsVolumeObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Attributes_List':
            obj_ = WindowsVolumeAttributesListType.factory()
            obj_.build(child_)
            self.set_Attributes_List(obj_)
        elif nodeName_ == 'Drive_Letter':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Drive_Letter(obj_)
        elif nodeName_ == 'Drive_Type':
            obj_ = WindowsDriveType.factory()
            obj_.build(child_)
            self.set_Drive_Type(obj_)
        super(WindowsVolumeObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class WindowsVolumeObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Device_Path': cybox_common.StringObjectPropertyType,
    'Time': cybox_common.TimeType,
    'Errors': cybox_common.ErrorsType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'File_System_Flag_List': volume_object.FileSystemFlagListType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Compiler': cybox_common.CompilerType,
    'Functions': cybox_common.FunctionsType,
    'File_System_Flag': volume_object.VolumeFileSystemFlagType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Type': cybox_common.ControlledVocabularyStringType,
    'Compilers': cybox_common.CompilersType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'Serial_Number': cybox_common.StringObjectPropertyType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Reference_Description': cybox_common.StructuredTextType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Drive_Letter': cybox_common.StringObjectPropertyType,
    'Import': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Sectors_Per_Allocation_Unit': cybox_common.UnsignedIntegerObjectPropertyType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Total_Allocation_Units': cybox_common.UnsignedLongObjectPropertyType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Language': cybox_common.StringObjectPropertyType,
    'Creation_Time': cybox_common.DateTimeObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'Contributors': cybox_common.PersonnelType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Libraries': cybox_common.LibrariesType,
    'Function': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'Volume': volume_object.VolumeObjectType,
    'Address': cybox_common.HexBinaryObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Name': cybox_common.StringObjectPropertyType,
    'File_System_Type': cybox_common.StringObjectPropertyType,
    'Actual_Available_Allocation_Units': cybox_common.UnsignedLongObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Data_Size': cybox_common.DataSizeType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'Contributor': cybox_common.ContributorType,
    'Bytes_Per_Sector': cybox_common.PositiveIntegerObjectPropertyType,
    'Tools': cybox_common.ToolsInformationType,
    'Tool': cybox_common.ToolInformationType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Volume'
        rootClass = WindowsVolumeObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Volume'
        rootClass = WindowsVolumeObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Volume'
        rootClass = WindowsVolumeObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="Windows_Volume",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "WindowsVolumeObjectType",
    "WindowsVolumeAttributesListType",
    "WindowsDriveType",
    "WindowsVolumeAttributeType"
    ]



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.win_service_object import ServiceDescriptionList, WinService
from cybox.test import EntityTestCase
from cybox.test.common.hash_test import EMPTY_MD5
from cybox.test.objects import ObjectTestCase


class TestServiceDescriptionList(EntityTestCase, unittest.TestCase):
    klass = ServiceDescriptionList

    _full_dict = [
        u('Provides infrastructure support for deploying Store applications. This service is started on demand and if disabled Store applications will not be deployed to the system, and may not function properly.'),
        u('Description 2'),
    ]


class TestWinService(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsServiceObjectType"
    klass = WinService

    _full_dict = {
        'service_dll_signature_exists': True,
        'service_dll_signature_verified': True,
        'description_list': TestServiceDescriptionList._full_dict,
        'display_name': u('AppXSvc'),
        'group_name': u('appx'),
        'service_name': u('AppX Deployment Service (AppXSVC)'),
        'service_dll': u('appxsvc.dll'),
        'service_dll_certificate_issuer': u('Microsoft Corporation'),
        'service_dll_certificate_subject': u('Frabrikam'),
        'service_dll_hashes': [
            {'type': u("MD5"), 'simple_hash_value': EMPTY_MD5}
        ],
        'service_dll_signature_description': u('Something'),
        'startup_command_line': u('C:\WINDOWS\system32\svchost.exe -k wsappx -p'),
        'startup_type': u('SERVICE_AUTO_START'),
        'service_status': u('SERVICE_PAUSED'),
        'service_type': u('SERVICE_KERNEL_DRIVER'),
        'started_as': u('Local Service'),
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.win_executable_file_object import (
    DataDirectory, DOSHeader, Entropy, PEBuildInformation, PEChecksum,
    PEDataDirectoryStruct, PEExportedFunction, PEExportedFunctions,
    PEExports, PEFileHeader, PEHeaders, PEImport, PEImportedFunction,
    PEImportedFunctions, PEImportList, PEOptionalHeader, PEResource,
    PEResourceList, PESection, PESectionHeaderStruct, PESectionList,
    PEVersionInfoResource, WinExecutableFile
)
from cybox.test import EntityTestCase
from cybox.test.common.hash_test import EMPTY_MD5
from cybox.test.common.digital_signature_test import TestDigitalSignature
from cybox.test.objects import ObjectTestCase


class TestPEDataDirectoryStruct(EntityTestCase, unittest.TestCase):
    klass = PEDataDirectoryStruct

    _full_dict = {
        'virtual_address': u('0xFBCE'),
        'size': 23040,
    }


class TestDataDirectory(EntityTestCase, unittest.TestCase):
    klass = DataDirectory

    _full_dict = {
        'export_table': TestPEDataDirectoryStruct._full_dict,
        'import_table': TestPEDataDirectoryStruct._full_dict,
        'resource_table': TestPEDataDirectoryStruct._full_dict,
        'exception_table': TestPEDataDirectoryStruct._full_dict,
        'certificate_table': TestPEDataDirectoryStruct._full_dict,
        'base_relocation_table': TestPEDataDirectoryStruct._full_dict,
        'debug': TestPEDataDirectoryStruct._full_dict,
        'architecture': TestPEDataDirectoryStruct._full_dict,
        'global_ptr': TestPEDataDirectoryStruct._full_dict,
        'tls_table': TestPEDataDirectoryStruct._full_dict,
        'load_config_table': TestPEDataDirectoryStruct._full_dict,
        'bound_import': TestPEDataDirectoryStruct._full_dict,
        'import_address_table': TestPEDataDirectoryStruct._full_dict,
        'delay_import_descriptor': TestPEDataDirectoryStruct._full_dict,
        'clr_runtime_header': TestPEDataDirectoryStruct._full_dict,
        'reserved': TestPEDataDirectoryStruct._full_dict,
    }


class TestDOSHeader(EntityTestCase, unittest.TestCase):
    klass = DOSHeader

    _full_dict = {
        'e_magic': u('0x1'),
        'e_cblp': u('0x2'),
        'e_cp': u('0x3'),
        'e_crlc': u('0x4'),
        'e_cparhdr': u('0x5'),
        'e_minalloc': u('0x6'),
        'e_maxalloc': u('0x7'),
        'e_ss': u('0x8'),
        'e_sp': u('0x9'),
        'e_csum': u('0x10'),
        'e_ip': u('0x11'),
        'e_cs': u('0x12'),
        'e_lfarlc': u('0x13'),
        'e_ovro': u('0x14'),
        'e_oemid': u('0x15'),
        'e_oeminfo': u('0x16'),
        'reserved2': u('0x17'),
        'e_lfanew': u('0x18'),
        'hashes': [
            {'type': u("MD5"), 'simple_hash_value': EMPTY_MD5},
        ],
        'reserved1': [u('0x20'), u('0x21'), u('0x22')],
    }


class TestEntropy(EntityTestCase, unittest.TestCase):
    klass = Entropy

    _full_dict = {
        'value': 3.45673,
        'min': 2.23467,
        'max': 4.42346,
    }


class TestPEBuildInformation(EntityTestCase, unittest.TestCase):
    klass = PEBuildInformation

    _full_dict = {
        'linker_name': u('lld'),
        'linker_version': u('11'),
        'compiler_name': u('GNU GCC'),
        'compiler_version': u('7.8.4'),
    }


class TestPEChecksum(EntityTestCase, unittest.TestCase):
    klass = PEChecksum

    _full_dict = {
        'pe_computed_api': 120,
        'pe_file_api': 12800,
        'pe_file_raw': 22500,
    }


class TestPEExportedFunction(EntityTestCase, unittest.TestCase):
    klass = PEExportedFunction

    _full_dict = {
        'function_name': u('important_calculation'),
        'entry_point': u('0x000ECB99'),
        'ordinal': 1,
    }


class TestPEExportedFunctions(EntityTestCase, unittest.TestCase):
    klass = PEExportedFunctions

    _full_dict = [
        TestPEExportedFunction._full_dict,
        TestPEExportedFunction._full_dict,
    ]


class TestPEExports(EntityTestCase, unittest.TestCase):
    klass = PEExports

    _full_dict = {
        'name': u('PE Export Name'),
        'exported_functions': TestPEExportedFunctions._full_dict,
        'number_of_functions': 250,
        'exports_time_stamp': '2013-08-08T15:15:15-04:00',
        'number_of_addresses': 55000,
        'number_of_names': 10,
    }


class TestPEFileHeader(EntityTestCase, unittest.TestCase):
    klass = PEFileHeader

    _full_dict = {
        'machine': u('0xAA'),
        'number_of_sections': 3500,
        'time_date_stamp': u('0x000000005E619BF6'),
        'pointer_to_symbol_table': u('0x00023FEBB'),
        'number_of_symbols': 300,
        'size_of_optional_header': u('0x000CE'),
        'characteristics': u('0x4C'),
        'hashes': [
            {'type': u("MD5"), 'simple_hash_value': EMPTY_MD5},
        ],
    }


class TestPEImportedFunction(EntityTestCase, unittest.TestCase):
    klass = PEImportedFunction

    _full_dict = {
        'function_name': u('important_calculation_2'),
        'hint': u('0xEEFB'),
        'ordinal': 2,
        'bound': u('0x213EFB'),
        'virtual_address': u('0x000AB12'),
    }


class TestPEImportedFunctions(EntityTestCase, unittest.TestCase):
    klass = PEImportedFunctions

    _full_dict = [
        TestPEImportedFunction._full_dict,
        TestPEImportedFunction._full_dict,
    ]


class TestPEImport(EntityTestCase, unittest.TestCase):
    klass = PEImport

    _full_dict = {
        'delay_load': True,
        'initially_visible': True,
        'file_name': u('KERNEL32.GetModuleHandle'),
        'imported_functions': TestPEImportedFunctions._full_dict,
        'virtual_address': u('0x000000007B0D0000'),
    }


class TestPEOptionalHeader(EntityTestCase, unittest.TestCase):
    klass = PEOptionalHeader

    _full_dict = {
        'magic': u('0xECBA'),
        'major_linker_version': u('0x11'),
        'minor_linker_version': u('0x0'),
        'size_of_code': u('0x57E4'),
        'size_of_initialized_data': u('0x8CBA'),
        'size_of_uninitialized_data': u('0xEECB'),
        'address_of_entry_point': u('0x2345'),
        'base_of_code': u('0x0B35'),
        'base_of_data': u('0x0234'),
        'image_base': u('0xFFEE'),
        'section_alignment': u('0x00BC'),
        'file_alignment': u('0xBBEE'),
        'major_os_version': u('0x0011'),
        'minor_os_version': u('0x1001'),
        'major_image_version': u('0xEFD2'),
        'minor_image_version': u('0x328F'),
        'major_subsystem_version': u('0x5544'),
        'minor_subsystem_version': u('0x40FB'),
        'win32_version_value': u('0xAB0E'),
        'size_of_image': u('0xDE55'),
        'size_of_headers': u('0x34FF'),
        'checksum': u('0xABCDEF123456'),
        'subsystem': u('0xBC39012'),
        'dll_characteristics': u('0xDB45E'),
        'size_of_stack_reserve': u('0xAABB'),
        'size_of_stack_commit': u('0xEFFC'),
        'size_of_heap_reserve': u('0x2345'),
        'size_of_heap_commit': u('0xDEED'),
        'loader_flags': u('0xBEAD'),
        'number_of_rva_and_sizes': u('0x9823'),
        'data_directory': TestDataDirectory._full_dict,
        'hashes': [
            {
                'type': u("MD5"),
                'simple_hash_value': EMPTY_MD5,
            }
        ],
    }


class TestPEHeaders(EntityTestCase, unittest.TestCase):
    klass = PEHeaders

    _full_dict = {
        'dos_header': TestDOSHeader._full_dict,
        'signature': u('0x00000000ABAB045D'),
        'file_header': TestPEFileHeader._full_dict,
        'optional_header': TestPEOptionalHeader._full_dict,
        'entropy': TestEntropy._full_dict,
        'hashes': [
            {'type': u("MD5"), 'simple_hash_value': EMPTY_MD5},
        ],
    }


class TestPEImportList(EntityTestCase, unittest.TestCase):
    klass = PEImportList

    _full_dict = [
        TestPEImport._full_dict,
        TestPEImport._full_dict,
    ]


class TestPEResource(EntityTestCase, unittest.TestCase):
    klass = PEResource

    _full_dict = {
        'type': u('TypeLib'),
        'name': u('Some type lib'),
        'size': 12800,
        'virtual_address': u('0xDEADBEEF'),
        'language': u('en'),
        'sub_language': u('en_us'),
        'hashes': [
            {
                'type': u("MD5"),
                'simple_hash_value': EMPTY_MD5,
            }
        ],
        'data': u('Z2xlA2NvbQAADwABwAwADwABAAACKAAKACgFc210cDTADMAMAA8AAQAAAigACgAK'),
    }


class TestPEVersionInfoResource(EntityTestCase, unittest.TestCase):
    klass = PEVersionInfoResource

    _full_dict = {
        'comments': u('Some PE Comments'),
        'companyname': u('Fabrikam Corp'),
        'filedescription': u('Some File Description'),
        'fileversion': u('V1.0.0.1'),
        'internalname': u('Frabik'),
        'langid': u('0x0409'),
        'legalcopyright': u('(C)2011 Fabrikam Corp'),
        'legaltrademarks': u('Frabrik is a trademark of Fabrikam'),
        'originalfilename': u('testfile.sys'),
        'privatebuild': u('4352.54'),
        'productname': u('Fabrikam Driver'),
        'productversion': u('V1.0.0.1'),
        'specialbuild': u('V1.0.0.1a54'),
        'xsi:type': 'WinExecutableFileObj:PEVersionInfoResourceType',
    }


class TestPEResourceList(EntityTestCase, unittest.TestCase):
    klass = PEResourceList

    mixed_dict = TestPEVersionInfoResource._full_dict
    mixed_dict.update(TestPEResource._full_dict)

    _full_dict = [
        TestPEResource._full_dict,
        TestPEVersionInfoResource._full_dict,
        mixed_dict,
    ]


class TestPESectionHeaderStruct(EntityTestCase, unittest.TestCase):
    klass = PESectionHeaderStruct

    _full_dict = {
        'name': u('Section Header Name'),
        'virtual_size': u('0xECBA5'),
        'virtual_address': u('0x00000002EF599'),
        'size_of_raw_data': u('0xFEB5503'),
        'pointer_to_raw_data': u('0x00FB'),
        'pointer_to_relocations': u('0xEEBC'),
        'pointer_to_linenumbers': u('0x00000635D'),
        'number_of_relocations': 5,
        'number_of_linenumbers': 200,
        'characteristics': u('0xABCD'),
    }


class TestPESection(EntityTestCase, unittest.TestCase):
    klass = PESection

    _full_dict = {
        'section_header': TestPESectionHeaderStruct._full_dict,
        'data_hashes': [
            {'type': u("MD5"), 'simple_hash_value': EMPTY_MD5},
        ],
        'entropy': TestEntropy._full_dict,
        'header_hashes': [
            {'type': u("MD5"), 'simple_hash_value': EMPTY_MD5},
            {'type': u("MD5"), 'simple_hash_value': EMPTY_MD5},
        ],
    }


class TestPESectionList(EntityTestCase, unittest.TestCase):
    klass = PESectionList

    _full_dict = [
        TestPESection._full_dict,
        TestPESection._full_dict,
    ]


class TestWinExecutableFile(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsExecutableFileObjectType"
    klass = WinExecutableFile

    _full_dict = {
        'build_information': TestPEBuildInformation._full_dict,
        'digital_signature': TestDigitalSignature._full_dict,
        'exports': TestPEExports._full_dict,
        'extraneous_bytes': 345,
        'headers': TestPEHeaders._full_dict,
        'imports': TestPEImportList._full_dict,
        'pe_checksum': TestPEChecksum._full_dict,
        'resources': TestPEResourceList._full_dict,
        'sections': TestPESectionList._full_dict,
        'type': u('Executable'),
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities, fields

import cybox.bindings.cybox_common as common_binding


class Metadata(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.MetadataType
    _namespace = 'http://cybox.mitre.org/common-2'

    type_ = fields.TypedField("type_", key_name="type")
    value = fields.TypedField("Value")
    subdatum = fields.TypedField("SubDatum", type_="cybox.common.metadata.Metadata", multiple=True)



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.unix_pipe_object as unix_pipe_binding
from cybox.common import String
from cybox.objects.pipe_object import Pipe


class UnixPipe(Pipe):
    _binding = unix_pipe_binding
    _binding_class = unix_pipe_binding.UnixPipeObjectType
    _namespace = "http://cybox.mitre.org/objects#UnixPipeObject-2"
    _XSI_NS = "UnixPipeObj"
    _XSI_TYPE = "UnixPipeObjectType"

    permission_mode = fields.TypedField("Permission_Mode", String)



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities, fields

import cybox
import cybox.bindings.cybox_core as core_binding
from cybox.common import DataSegment


class DefinedEffectFactory(entities.EntityFactory):
    @classmethod
    def entity_class(cls, key):
        return cybox.lookup_extension(key, default=DefinedEffect)


class DefinedEffect(entities.Entity):
    _binding = core_binding
    _binding_class = core_binding.DefinedEffectType
    _namespace = 'http://cybox.mitre.org/cybox-2'
    _XSI_TYPE = None    # overridden by subclasses

    TERM_STATE_CHANGED = 'State_Changed'
    TERM_DATA_READ = 'Data_Read'
    TERM_DATA_WRITTEN = 'Data_Written'
    TERM_DATA_RECEIVED = 'Data_Received'
    TERM_PROPERTIES_READ = 'Properties_Read'
    TERM_PROPERTIES_ENUMERATED = 'Properties_Enumerated'
    TERM_VALUES_ENUMERATED = 'Values_Enumerated'
    TERM_CONTROLCODE_SENT = 'ControlCode_Sent'

    effect_type = fields.TypedField("effect_type")

    def to_dict(self):
        d = super(DefinedEffect, self).to_dict()

        if self._XSI_TYPE:
            d["xsi:type"] = self._XSI_TYPE

        return d

    @staticmethod
    def lookup_class(xsi_type):
        return cybox.lookup_extension(xsi_type, default=DefinedEffect)


@cybox.register_extension
class StateChangeEffect(DefinedEffect):
    _binding = core_binding
    _binding_class = core_binding.StateChangeEffectType
    _namespace = 'http://cybox.mitre.org/cybox-2'
    _XSI_TYPE = "cybox:StateChangeEffectType"

    old_object = fields.TypedField("Old_Object", type_="cybox.core.object.Object")
    new_object = fields.TypedField("New_Object", type_="cybox.core.object.Object")


@cybox.register_extension
class DataReadEffect(DefinedEffect):
    _binding = core_binding
    _binding_class = core_binding.DataReadEffectType
    _namespace = 'http://cybox.mitre.org/cybox-2'
    _XSI_TYPE = "cybox:DataReadEffectType"

    data = fields.TypedField("Data", DataSegment)


@cybox.register_extension
class DataWrittenEffect(DefinedEffect):
    _binding = core_binding
    _binding_class = core_binding.DataWrittenEffectType
    _namespace = 'http://cybox.mitre.org/cybox-2'
    _XSI_TYPE = "cybox:DataWrittenEffectType"

    data = fields.TypedField("Data", DataSegment)


@cybox.register_extension
class DataSentEffect(DefinedEffect):
    _binding = core_binding
    _binding_class = core_binding.DataSentEffectType
    _namespace = 'http://cybox.mitre.org/cybox-2'
    _XSI_TYPE = "cybox:DataSentEffectType"

    data = fields.TypedField("Data", DataSegment)


@cybox.register_extension
class DataReceivedEffect(DefinedEffect):
    _binding = core_binding
    _binding_class = core_binding.DataReceivedEffectType
    _namespace = 'http://cybox.mitre.org/cybox-2'
    _XSI_TYPE = "cybox:DataReceivedEffectType"

    data = fields.TypedField("Data", DataSegment)


@cybox.register_extension
class PropertyReadEffect(DefinedEffect):
    _binding = core_binding
    _binding_class = core_binding.PropertyReadEffectType
    _namespace = 'http://cybox.mitre.org/cybox-2'
    _XSI_TYPE = "cybox:PropertyReadEffectType"

    name = fields.TypedField("Name")
    value = fields.TypedField("Value")


class Properties(entities.Entity):
    _binding = core_binding
    _binding_class = core_binding.PropertiesType
    _namespace = 'http://cybox.mitre.org/cybox-2'

    property_ = fields.TypedField("Property", multiple=True)


@cybox.register_extension
class PropertiesEnumeratedEffect(DefinedEffect):
    _binding = core_binding
    _binding_class = core_binding.PropertiesEnumeratedEffectType
    _namespace = 'http://cybox.mitre.org/cybox-2'
    _XSI_TYPE = "cybox:PropertiesEnumeratedEffectType"

    properties = fields.TypedField("Properties", Properties)


class Values(entities.Entity):
    _binding = core_binding
    _binding_class = core_binding.ValuesType
    _namespace = 'http://cybox.mitre.org/cybox-2'

    value = fields.TypedField("Value", multiple=True)


@cybox.register_extension
class ValuesEnumeratedEffect(DefinedEffect):
    _binding = core_binding
    _binding_class = core_binding.ValuesEnumeratedEffectType
    _namespace = 'http://cybox.mitre.org/cybox-2'
    _XSI_TYPE = "cybox:ValuesEnumeratedEffectType"

    values = fields.TypedField("Values", Values)


@cybox.register_extension
class SendControlCodeEffect(DefinedEffect):
    _binding = core_binding
    _binding_class = core_binding.SendControlCodeEffectType
    _namespace = 'http://cybox.mitre.org/cybox-2'
    _XSI_TYPE = "cybox:SendControlCodeEffectType"

    control_code = fields.TypedField("Control_Code")



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.win_mutex_object import WinMutex
from cybox.test.objects import ObjectTestCase
from cybox.test.objects.win_handle_test import TestWinHandle


class TestWinMutex(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsMutexObjectType"
    klass = WinMutex

    _full_dict = {
        'handle': TestWinHandle._full_dict,
        'security_attributes': u("S-1-5-21-3623811015-3361044348-30300820-1013"),
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities, fields

import cybox.bindings.network_flow_object as network_flow_binding
from cybox.common import (
    BaseProperty, HexBinary, Integer, ObjectProperties, PlatformSpecification,
    PositiveInteger, String
)
from cybox.objects.address_object import Address
from cybox.objects.network_packet_object import TCPFlags
from cybox.objects.socket_address_object import SocketAddress


class NetflowV5FlowRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV5FlowRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    nexthop_ipv4_addr = fields.TypedField("Nexthop_IPv4_Addr", Address)
    packet_count = fields.TypedField("Packet_Count", Integer)
    byte_count = fields.TypedField("Byte_Count", Integer)
    sysuptime_start = fields.TypedField("SysUpTime_Start", Integer)
    sysuptime_end = fields.TypedField("SysUpTime_End", Integer)
    padding1 = fields.TypedField("Padding1", HexBinary)
    tcp_flags = fields.TypedField("TCP_Flags", HexBinary)
    src_autonomous_system = fields.TypedField("Src_Autonomous_System", Integer)
    dest_autonomous_system = fields.TypedField("Dest_Autonomous_System", Integer)
    src_ip_mask_bit_count = fields.TypedField("Src_IP_Mask_Bit_Count", String)
    dest_ip_mask_bit_count = fields.TypedField("Dest_IP_Mask_Bit_Count", String)
    padding2 = fields.TypedField("Padding2", HexBinary)


class NetflowV5FlowHeader(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV5FlowHeaderType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    version = fields.TypedField("Version", HexBinary)
    count = fields.TypedField("Count", Integer)
    sys_up_time = fields.TypedField("Sys_Up_Time", Integer)
    unix_secs = fields.TypedField("Unix_Secs", Integer)
    unix_nsecs = fields.TypedField("Unix_Nsecs", Integer)
    flow_sequence = fields.TypedField("Flow_Sequence", Integer)
    engine_type = fields.TypedField("Engine_Type", String)
    engine_id = fields.TypedField("Engine_ID", Integer)
    sampling_interval = fields.TypedField("Sampling_Interval", HexBinary)

    def __init__(self):
        super(NetflowV5FlowHeader, self).__init__()
        self.version = "05"


class NetflowV5Packet(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV5PacketType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    flow_header = fields.TypedField("Flow_Header", NetflowV5FlowHeader)
    flow_record = fields.TypedField("Flow_Record", NetflowV5FlowRecord, multiple=True)


class NetflowV9PacketHeader(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV9PacketHeaderType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    version = fields.TypedField("Version", HexBinary)
    record_count = fields.TypedField("Record_Count", Integer)
    sys_up_time = fields.TypedField("Sys_Up_Time", Integer)
    unix_secs = fields.TypedField("Unix_Secs", Integer)
    sequence_number = fields.TypedField("Sequence_Number", Integer)
    source_id = fields.TypedField("Source_ID", HexBinary)

    def __init__(self, version=None):
        super(NetflowV9PacketHeader, self).__init__()
        self.version = version or "09"


class NetflowV9ScopeField(BaseProperty):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV9ScopeFieldType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    TERM_SYSTEM = "System(1)"
    TERM_INTERFACE = "Interface(2)"
    TERM_LINE_CARD = "LineCard(3)"
    TERM_CACHE = "Cache(4)"
    TERM_TEMPLATE = "Template(5)"


class NetflowV9Field(BaseProperty):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV9FieldType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    TERM_IN_BYTES = "IN_BYTES(1)"
    TERM_IN_PKTS = "IN_PKTS(2)"
    TERM_FLOWS = "FLOWS(3)"
    TERM_PROTOCOL = "PROTOCOL(4)"
    TERM_TOS = "SRC_TOS(5)"
    TERM_TCP_FLAGS = "TCP_FLAGS(6)"
    TERM_L4_SRC_PORT = "L4_SRC_PORT(7)"
    TERM_IPV4_SRC_ADDR = "IPV4_SRC_ADDR(8)"
    TERM_SRC_MASK = "SRC_MASK(9)"
    TERM_INPUT_SNMP = "INPUT_SNMP(10)"
    TERM_L4_DST_PORT= "L4_DST_PORT(11)"
    TERM_IPV4_DST_ADDR = "IPV4_DST_ADDR(12)"
    TERM_DST_MASK= "DST_MASK(13)"
    TERM_OUTPUT_SNMP = "OUTPUT_SNMP(14)"
    TERM_IPV4_NEXT_HOP = "IPV4_NEXT_HOP(15)"
    TERM_SRC_AS = "SRC_AS(16)"
    TERM_DST_AS = "DST_AS(17)"
    TERM_BGP_IPV4_NEXT_HOP = "BGP_IPV4_NEXT_HOP(18)"
    TERM_MUL_DST_PKTS = "MUL_DST_PKTS(19)"
    TERM_MUL_DST_BYTES = "MUL_DST_BYTES(20)"


class NetflowV9TemplateRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV9TemplateRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    template_id = fields.TypedField("Template_ID", Integer)
    field_count = fields.TypedField("Field_Count", Integer)
    field_type = fields.TypedField("Field_Type", NetflowV9Field)
    field_length = fields.TypedField("Field_Length", HexBinary)


class NetflowV9TemplateFlowSet(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV9TemplateFlowSetType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    flow_set_id = fields.TypedField("Flow_Set_ID", HexBinary)
    length = fields.TypedField("Length", Integer)
    template_record = fields.TypedField("Template_Record", NetflowV9TemplateRecord, multiple=True)

    def __init__(self):
        super(NetflowV9TemplateFlowSet, self).__init__()
        self.flow_set_id = "00"


class NetflowV9OptionsTemplateRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV9OptionsTemplateRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    template_id = fields.TypedField("Template_ID", Integer)
    option_scope_length = fields.TypedField("Option_Scope_Length", HexBinary)
    option_length = fields.TypedField("Option_Length", HexBinary)
    scope_field_type = fields.TypedField("Scope_Field_Type", NetflowV9ScopeField)
    scope_field_length = fields.TypedField("Scope_Field_Length", HexBinary)
    option_field_type = fields.TypedField("Option_Field_Type", NetflowV9Field)
    option_field_length = fields.TypedField("Option_Field_Length", HexBinary)


class NetflowV9OptionsTemplateFlowSet(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV9OptionsTemplateFlowSetType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    flow_set_id = fields.TypedField("Flow_Set_ID", HexBinary)
    length = fields.TypedField("Length", Integer)
    options_template_record = fields.TypedField("Options_Template_Record", NetflowV9OptionsTemplateRecord, multiple=True)
    padding = fields.TypedField("Padding", HexBinary)

    def __init__(self):
        super(NetflowV9OptionsTemplateFlowSet, self).__init__()
        self.flow_set_id = "01"


class FlowCollectionElement(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.FlowCollectionElementType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    flow_record_field_value = fields.TypedField("Flow_Record_Field_Value", String, multiple=True)


class FlowDataRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.FlowDataRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    flow_record_collection_element = fields.TypedField("Flow_Record_Collection_Element", FlowCollectionElement, multiple=True)


class OptionCollectionElement(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.OptionCollectionElementType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    option_record_field_value = fields.TypedField("Option_Record_Field_Value", String, multiple=True)


class OptionsDataRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.OptionsDataRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    scope_field_value = fields.TypedField("Scope_Field_Value", String)
    option_record_collection_element = fields.TypedField("Option_Record_Collection_Element", OptionCollectionElement, multiple=True)


class NetflowV9DataRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV9DataRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    flow_data_record = fields.TypedField("Flow_Data_Record", FlowDataRecord, multiple=True)
    options_data_record = fields.TypedField("Options_Data_Record", OptionsDataRecord, multiple=True)


class NetflowV9DataFlowSet(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV9DataFlowSetType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    flow_set_id_template_id = fields.TypedField("Flow_Set_ID_Template_ID", Integer)
    length = fields.TypedField("Length", Integer)
    data_record = fields.TypedField("Data_Record", NetflowV9DataRecord, multiple=True)
    padding = fields.TypedField("Padding", HexBinary)


class NetflowV9FlowSet(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV9FlowSetType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    template_flow_set = fields.TypedField("Template_Flow_Set", NetflowV9TemplateFlowSet)
    options_template_flow_set = fields.TypedField("Options_Template_Flow_Set", NetflowV9OptionsTemplateFlowSet)
    data_flow_set = fields.TypedField("Data_Flow_Set", NetflowV9DataFlowSet)


class NetflowV9ExportPacket(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetflowV9ExportPacketType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    flow_header = fields.TypedField("Packet_Header", NetflowV9PacketHeader)
    flow_set = fields.TypedField("Flow_Set", NetflowV9FlowSet, multiple=True)


class SiLKSensorDirection(BaseProperty):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.SiLKDirectionType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    TERM_IN = "in"
    TERM_IN_WEB = "inweb"
    TERM_IN_NULL = "innull"
    TERM_OUT = "out"
    TERM_OUT_WEB = "outweb"
    TERM_OUT_NULL = "outnull"


class SiLKSensorClass(BaseProperty):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.SiLKSensorClassType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    TERM_ALL = "all"


class SiLKCountryCode(BaseProperty):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.SiLKCountryCodeType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"


class SiLKAddress(BaseProperty):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.SiLKAddressType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    TERM_NON_ROUTABLE = "non-routable (0)"
    TERM_INTERNAL = "internal(1)"
    TERM_EXTERNAL = "routable_external(2)"


class SiLKFlowAttributes(BaseProperty):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.SiLKAddressType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    TERM_F = "F (FIN flag)"
    TERM_T = "T (Timeout)"
    TERM_C = "C (Continuation)"


class SiLKSensorInfo(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.SiLKSensorInfoType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    sensor_id = fields.TypedField("Sensor_ID", String)
    class_ = fields.TypedField("Class", SiLKSensorClass)
    type_ = fields.TypedField("Type", SiLKSensorDirection)


class SiLKRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.SiLKRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    packet_count = fields.TypedField("Packet_Count", Integer)
    byte_count = fields.TypedField("Byte_Count", Integer)
    tcp_flags = fields.TypedField("TCP_Flags", HexBinary)
    start_time = fields.TypedField("Start_Time", Integer)
    duration = fields.TypedField("Duration", Integer)
    end_time = fields.TypedField("End_Time", Integer)
    sensor_info = fields.TypedField("Sensor_Info", SiLKSensorInfo)
    icmp_type = fields.TypedField("ICMP_Type", Integer)
    icmp_code = fields.TypedField("ICMP_Code", Integer)
    router_next_hop_ip = fields.TypedField("Router_Next_Hop_IP", Address)
    initial_tcp_flags = fields.TypedField("Initial_TCP_Flags", TCPFlags)
    session_tcp_flags = fields.TypedField("Session_TCP_Flags", HexBinary)
    flow_attributes = fields.TypedField("Flow_Attributes", SiLKFlowAttributes)
    flow_application = fields.TypedField("Flow_Application", String)
    src_ip_type = fields.TypedField("Src_IP_Type", SiLKAddress)
    dest_ip_type = fields.TypedField("Dest_IP_Type", SiLKAddress)
    src_country_code = fields.TypedField("Src_Country_Code", SiLKCountryCode)
    dest_country_code = fields.TypedField("Dest_Country_Code", SiLKCountryCode)
    src_mapname = fields.TypedField("Src_MAPNAME", String)
    dest_mapname = fields.TypedField("Dest_MAPNAME", String)


class IPFIXMessageHeader(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXMessageHeaderType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    version = fields.TypedField("Version", HexBinary)
    byte_length = fields.TypedField("Byte_Length", HexBinary)
    export_timestamp = fields.TypedField("Export_Timestamp", Integer)
    sequence_number = fields.TypedField("Sequence_Number", Integer)
    observation_domain_id = fields.TypedField("Observation_Domain_ID", Integer)

    def __init__(self):
        super(IPFIXMessageHeader, self).__init__()
        self.version = "0a"


class IPFIXSetHeader(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXSetHeaderType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    set_id = fields.TypedField("Set_ID", Integer)
    length = fields.TypedField("Length", Integer)


class IPFIXTemplateRecordHeader(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXTemplateRecordHeaderType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    template_id = fields.TypedField("Template_ID", Integer)
    field_count = fields.TypedField("Field_Count", HexBinary)


class IPFIXTemplateRecordFieldSpecifiers(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXTemplateRecordFieldSpecifiersType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    enterprise_bit = fields.TypedField("Enterprise_Bit")
    information_element_id = fields.TypedField("Information_Element_ID", String)
    field_length = fields.TypedField("Field_Length", String)
    enterprise_number = fields.TypedField("Enterprise_Number", String)


class IPFIXTemplateRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXTemplateRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    template_record_header = fields.TypedField("Template_Record_Header", IPFIXTemplateRecordHeader)
    field_specifier = fields.TypedField("Field_Specifier", IPFIXTemplateRecordFieldSpecifiers, multiple=True)


class IPFIXTemplateSet(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXTemplateSetType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    set_header = fields.TypedField("Set_Header", IPFIXSetHeader)
    template_record = fields.TypedField("Template_Record", IPFIXTemplateRecord, multiple=True)
    padding = fields.TypedField("Padding", HexBinary)


class IPFIXOptionsTemplateRecordHeader(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXOptionsTemplateRecordHeaderType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    template_id = fields.TypedField("Template_ID", Integer)
    field_count = fields.TypedField("Field_Count", HexBinary)
    scope_field_count = fields.TypedField("Scope_Field_Count", PositiveInteger)


class IPFIXOptionsTemplateRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXOptionsTemplateRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    options_template_record_header = fields.TypedField("Options_Template_Record_Header", IPFIXOptionsTemplateRecordHeader)
    field_specifier = fields.TypedField("Field_Specifier", IPFIXTemplateRecordFieldSpecifiers, multiple=True)


class IPFIXOptionsTemplateSet(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXOptionsTemplateSetType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    set_header = fields.TypedField("Set_Header", IPFIXSetHeader)
    options_template_record = fields.TypedField("Options_Template_Record", IPFIXOptionsTemplateRecord, multiple=True)
    padding = fields.TypedField("Padding", HexBinary)


class IPFIXDataRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXDataRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    field_value = fields.TypedField("Field_Value", String, multiple=True)


class IPFIXDataSet(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXDataSetType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    set_header = fields.TypedField("Set_Header", IPFIXSetHeader)
    data_record = fields.TypedField("Data_Record", IPFIXDataRecord, multiple=True)
    padding = fields.TypedField("Padding", HexBinary)


class IPFIXSet(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXSetType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    template_set = fields.TypedField("Template_Set", IPFIXTemplateSet)
    options_template_set = fields.TypedField("Options_Template_Set", IPFIXOptionsTemplateSet)
    data_set = fields.TypedField("Data_Set", IPFIXDataSet)


class IPFIXMessage(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.IPFIXMessageType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    message_header = fields.TypedField("Message_Header", IPFIXMessageHeader)
    set_ = fields.TypedField("Set", IPFIXSet, multiple=True)


class UnidirectionalRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.UnidirectionalRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    ipfix_message = fields.TypedField("IPFIX_Message", IPFIXMessage)
    netflowv9_export_packet = fields.TypedField("NetflowV9_Export_Packet", NetflowV9ExportPacket)
    netflowv5_packet = fields.TypedField("NetflowV5_Packet", NetflowV5Packet)
    silk_record = fields.TypedField("SiLK_Record", SiLKRecord)


class NetworkLayerInfo(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetworkLayerInfoType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    src_socket_address = fields.TypedField("Src_Socket_Address", SocketAddress)
    dest_socket_address = fields.TypedField("Dest_Socket_Address", SocketAddress)
    ip_protocol = fields.TypedField("IP_Protocol", String)


class NetworkFlowLabel(NetworkLayerInfo):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetworkFlowLabelType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    ingress_interface_index = fields.TypedField("Ingress_Interface_Index", Integer)
    egress_interface_index = fields.TypedField("Egress_Interface_Index", Integer)
    ip_type_of_service = fields.TypedField("IP_Type_Of_Service", HexBinary)


class YAFTCPFlow(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.YAFTCPFlowType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    tcp_sequence_number = fields.TypedField("TCP_Sequence_Number", Integer)
    initial_tcp_flags = fields.TypedField("Initial_TCP_Flags", TCPFlags)
    union_tcp_flags = fields.TypedField("Union_TCP_Flags", HexBinary)


class YAFFlow(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.YAFFlowType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    flow_start_milliseconds = fields.TypedField("Flow_Start_Milliseconds", Integer)
    flow_end_milliseconds = fields.TypedField("Flow_End_Milliseconds", Integer)
    octet_total_count = fields.TypedField("Octet_Total_Count", Integer)
    packet_total_count = fields.TypedField("Packet_Total_Count", Integer)
    flow_end_reason = fields.TypedField("Flow_End_Reason", HexBinary)
    silk_app_label = fields.TypedField("SiLK_App_Label", Integer)
    payload_entropy = fields.TypedField("Payload_Entropy", Integer)
    ml_app_label = fields.TypedField("ML_App_Label", HexBinary)
    tcp_flow = fields.TypedField("TCP_Flow", YAFTCPFlow)
    vlan_id_mac_addr = fields.TypedField("Vlan_ID_MAC_Addr", Address)
    passive_os_fingerprinting = fields.TypedField("Passive_OS_Fingerprinting", PlatformSpecification)
    first_packet_banner = fields.TypedField("First_Packet_Banner", HexBinary)
    second_packet_banner = fields.TypedField("Second_Packet_Banner", HexBinary)
    n_bytes_payload = fields.TypedField("N_Bytes_Payload", HexBinary)


class YAFReverseFlow(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.YAFReverseFlowType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    reverse_octet_total_count = fields.TypedField("Reverse_Octet_Total_Count", Integer)
    reverse_packet_total_count = fields.TypedField("Reverse_Packet_Total_Count", Integer)
    reverse_payload_entropy = fields.TypedField("Reverse_Payload_Entropy", Integer)
    reverse_flow_delta_milliseconds = fields.TypedField("Reverse_Flow_Delta_Milliseconds", Integer)
    tcp_reverse_flow = fields.TypedField("TCP_Reverse_Flow", YAFTCPFlow)
    reverse_vlan_id_mac_addr = fields.TypedField("Reverse_Vlan_ID_MAC_Addr", Address)
    reverse_passive_os_fingerprinting = fields.TypedField("Reverse_Passive_OS_Fingerprinting", PlatformSpecification)
    reverse_first_packet = fields.TypedField("Reverse_First_Packet", HexBinary)
    reverse_n_bytes_payload = fields.TypedField("Reverse_N_Bytes_Payload", HexBinary)


class YAFRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.YAFRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    flow = fields.TypedField("Flow", YAFFlow)
    reverse_flow = fields.TypedField("Reverse_Flow", YAFReverseFlow)


class BidirectionalRecord(entities.Entity):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.BidirectionalRecordType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"

    yaf_record = fields.TypedField("YAF_Record", YAFRecord)


class NetworkFlow(ObjectProperties):
    _binding = network_flow_binding
    _binding_class = network_flow_binding.NetworkFlowObjectType
    _namespace = "http://cybox.mitre.org/objects#NetworkFlowObject-2"
    _XSI_NS = "NetFlowObj"
    _XSI_TYPE = "NetworkFlowObjectType"

    network_flow_label = fields.TypedField("Network_Flow_Label", NetworkFlowLabel)
    unidirectional_flow_record = fields.TypedField("Unidirectional_Flow_Record", UnidirectionalRecord)
    bidirectional_flow_record = fields.TypedField("Bidirectional_Flow_Record", BidirectionalRecord)



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities, fields

import cybox.bindings.cybox_common as common_binding


class ConfigurationSetting(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.ConfigurationSettingType
    _namespace = 'http://cybox.mitre.org/common-2'

    item_name = fields.TypedField("Item_Name")
    item_value = fields.TypedField("Item_Value")
    item_type = fields.TypedField("Item_Type")
    item_description = fields.TypedField("Item_Description")


class ConfigurationSettings(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.ConfigurationSettingsType
    _namespace = 'http://cybox.mitre.org/common-2'

    configuration_setting = fields.TypedField("Configuration_Setting", ConfigurationSetting, multiple=True)



# Copyright (c) 2020, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.win_process_object import MemorySectionList, StartupInfo, WinProcess
from cybox.test import EntityTestCase
from cybox.test.objects import ObjectTestCase
from cybox.test.objects.memory_test import TestMemory
from cybox.test.objects.win_handle_test import TestWinHandle


class TestMemorySectionList(EntityTestCase, unittest.TestCase):
    klass = MemorySectionList

    _full_dict = [
        TestMemory._full_dict,
        TestMemory._full_dict
    ]


class TestStartupInfo(EntityTestCase, unittest.TestCase):
    klass = StartupInfo

    _full_dict = {
        'lpdesktop': u('Desktop Title'),
        'lptitle': u('Program Title'),
        'dwx': 0,
        'dwy': 0,
        'dwxsize': 800,
        'dwysize': 600,
        'dwxcountchars': 800,
        'dwycountchars': 600,
        'dwfillattribute': 1,
        'dwflags': 0,
        'wshowwindow': 1,
        'hstdinput': TestWinHandle._full_dict,
        'hstdoutput': TestWinHandle._full_dict,
        'hstderror': TestWinHandle._full_dict,
    }


class TestWinProcess(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsProcessObjectType"
    klass = WinProcess

    _full_dict = {
        'aslr_enabled': True,
        'dep_enabled': False,
        'priority': u('normal'),
        'section_list': TestMemorySectionList._full_dict,
        'security_id': u("S-1-5-21-3623811015-3361044348-30300820-1013"),
        'startup_info': TestStartupInfo._full_dict,
        'security_type': u("SidTypeUser"),
        'window_title': u('Example Title'),
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2010, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities, fields

import cybox
import cybox.bindings.cybox_common as common_binding
from cybox.common import (
    CompensationModel, Errors, ExecutionEnvironment, HashList, Metadata,
    StructuredText
)
from cybox.common.vocabs import VocabField, ToolType


class ToolSpecificDataFactory(entities.EntityFactory):
    @classmethod
    def entity_class(cls, key):
        return cybox.lookup_extension(key, default=ToolSpecificData)


class ToolSpecificData(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.ToolSpecificDataType
    _namespace = 'http://cybox.mitre.org/common-2'
    _XSI_TYPE = None    # overridden by subclasses

    def to_dict(self):
        d = super(ToolSpecificData, self).to_dict()

        if self._XSI_TYPE:
            d["xsi:type"] = self._XSI_TYPE

        return d

    @staticmethod
    def lookup_class(xsi_type):
        return cybox.lookup_extension(xsi_type, default=ToolSpecificData)


class ToolReference(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.ToolReferenceType
    _namespace = 'http://cybox.mitre.org/common-2'

    TERM_DOCUMENTATION = "Documentation"
    TERM_SOURCE = "Source"
    TERM_DOWNLOAD = "Download"
    TERM_EXECUTE = "Execute"
    TERM_OTHER = "Other"

    reference_type = fields.TypedField("reference_type")
    value = fields.TypedField("valueOf_", key_name="value")


class ToolReferences(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.ToolReferencesType
    _namespace = 'http://cybox.mitre.org/common-2'

    reference = fields.TypedField("Reference", ToolReference, multiple=True)


class ToolConfiguration(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.ToolConfigurationType
    _namespace = 'http://cybox.mitre.org/common-2'

    configuration_settings = fields.TypedField("Configuration_Settings")
    dependencies = fields.TypedField("Dependencies")
    usage_context_assumptions = fields.TypedField("Usage_Context_Assumptions")
    internationalization_settings = fields.TypedField("Internationalization_Settings")
    build_information = fields.TypedField("Build_Information")


class ToolInformation(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.ToolInformationType
    _namespace = 'http://cybox.mitre.org/common-2'

    id_ = fields.IdField("id")
    idref = fields.IdrefField("idref")
    name = fields.TypedField("Name")
    type_ = VocabField("Type", ToolType, multiple=True)
    description = fields.TypedField("Description", StructuredText)
    references = fields.TypedField("References", ToolReferences)
    vendor = fields.TypedField("Vendor")
    version = fields.TypedField("Version")
    service_pack = fields.TypedField("Service_Pack")
    tool_specific_data = fields.TypedField("Tool_Specific_Data", ToolSpecificData, factory=ToolSpecificDataFactory)
    tool_hashes = fields.TypedField("Tool_Hashes", HashList)
    tool_configuration = fields.TypedField("Tool_Configuration", ToolConfiguration)
    execution_environment = fields.TypedField("Execution_Environment", ExecutionEnvironment)
    errors = fields.TypedField("Errors", Errors)
    metadata = fields.TypedField("Metadata", Metadata, multiple=True)
    compensation_model = fields.TypedField("Compensation_Model", CompensationModel)

    def __init__(self, tool_name=None, tool_vendor=None):
        super(ToolInformation, self).__init__()
        self.name = tool_name
        self.vendor = tool_vendor


class ToolInformationList(entities.EntityList):
    _binding_class = common_binding.ToolsInformationType
    _namespace = 'http://cybox.mitre.org/common-2'

    tool = fields.TypedField("Tool", ToolInformation, multiple=True)
